import { SortDirections } from '../models/sort';
import { DeepPartial, IRecord, TypeOf, Upsert } from './global';
import './object';
import './reflect';
type FilterDelegate<T> = (item: T, index: number) => boolean;
type SimpleMapDelegate<T, V = any> = (item: T) => V;
type MapDelegate<T, V = any> = (item: T, index: number) => V;
type IfElsePrimitive<T, U> = T extends string | number | boolean ? T : U;
type UpdatableItem<T, TKey extends keyof T = keyof T> = IfElsePrimitive<T, Upsert<T & IRecord, TKey>>;
type UpsertableItem<T, TKey extends keyof T = keyof T> = IfElsePrimitive<T, UpdatableItem<T, TKey>>;
type UpdateDelegate<T> = MapDelegate<T, DeepPartial<T>>;
type CalculationDelegate<T> = (item: T, index: number, prevItem: T, nextItem: T) => number;
type DiffMatcherDelegate<T, P> = (sourceItem: T, targetItem: P, sourceIndex: number, targetIndex: number) => boolean;
export interface IArrayOrderByConfig<T> {
    direction?: SortDirections;
    delegate: SimpleMapDelegate<T>;
}
export interface IMatchedArrayDiffItem<T, P> {
    sourceItem: T;
    targetItem: P;
}
export interface IArrayDiff<T, P> {
    added: P[];
    removed: T[];
    matched: IMatchedArrayDiffItem<T, P>[];
}
export declare const MergeWithUpdateOperations: {
    UseTargetIfDifferent: (item1: any, item2: any) => any;
    KeepSource: (item1: any) => any;
    KeepTarget: (_item1: any, item2: any) => any;
    Merge: (item1: any, item2: any) => any;
};
export interface ISyncWithOptions<T, P> {
    matchBy?(item1: T, item2: P): boolean;
    updateMatched?(item1: T, item2: P, item1Index: number, item2Index: number): T;
    updateUnmatched?(item1: T): T;
    createBy?(item2: P): T;
}
export interface IMergeWithOptions<T, P> extends ISyncWithOptions<T, P> {
    removeUnmatched?: boolean;
    addNew?: boolean;
    matchOrder?: boolean;
}
export declare class ArrayExtensions<T> {
    mapMany<V>(map: MapDelegate<T, V[]>): V[];
    ofType<V>(type: TypeOf<V>): V[];
    ofType(type: 'string'): string[];
    ofType(type: 'number'): number[];
    cast<V>(): V[];
    singleOrDefault(): T;
    singleOrDefault(filter: FilterDelegate<T>): T;
    firstOrDefault(): T;
    firstOrDefault(filter: FilterDelegate<T>): T;
    lastOrDefault(): T;
    lastOrDefault(filter: FilterDelegate<T>): T;
    clone(): T[];
    remove(item: T): T[];
    removeByFilter(filter: FilterDelegate<T>): T[];
    removeById(id: string): T[];
    indexOfId(id: string): number;
    findById(id: string): T;
    upsert(item: UpsertableItem<T>): T[];
    upsert(item: UpsertableItem<T>, index: number): T[];
    upsert(item: UpsertableItem<T>, index: number): T[];
    upsertMany(this: T[], items: T[], newIndex?: number): T[];
    replace(item: T): T[];
    replace(item: T, index: number): T[];
    replaceMany(items: T[]): T[];
    replaceMany(items: T[], index: number): T[];
    update(filter: FilterDelegate<T>, update: UpdateDelegate<T>): T[];
    insert(item: T): T[];
    insert(item: T, index: number): T[];
    chunk(size: number): T[][];
    clear(): void;
    move(from: number, to: number): T[];
    orderBy<R>(sorterDelegate: SimpleMapDelegate<T, R>): T[];
    orderBy<R>(sorterDelegate: SimpleMapDelegate<T, R>, direction: SortDirections): T[];
    orderBy(config: IArrayOrderByConfig<T>[]): T[];
    removeNull(): T[];
    except(array: UpdatableItem<T>[]): T[];
    distinct(): T[];
    distinct<V>(delegate: MapDelegate<T, V>): T[];
    equals(array: T[]): boolean;
    equals(array: T[], ignoreOrder: boolean): boolean;
    sum(): number;
    sum(delegate: CalculationDelegate<T>): number;
    min(): number;
    min(delegate: CalculationDelegate<T>): number;
    max(): number;
    max(delegate: CalculationDelegate<T>): number;
    average(): number;
    average(delegate: CalculationDelegate<T>): number;
    absorb(array: T[]): T[];
    any(): T;
    takeUntil(this: T[], delegate: FilterDelegate<T>): T[];
    takeUntil(this: T[], delegate: FilterDelegate<T>, andIncluding: boolean): T[];
    diff<P extends IRecord>(items: P[]): IArrayDiff<T, P>;
    diff<P>(items: P[], matcher: DiffMatcherDelegate<T, P>): IArrayDiff<T, P>;
    ids(): string[];
    mergeWith<P>(items: P[]): T[];
    mergeWith<P>(items: P[], options: IMergeWithOptions<T, P>): T[];
    syncWith<P>(items: P[]): T[];
    syncWith<P>(items: P[], options: IMergeWithOptions<T, P>): T[];
    take(count: number): T[];
}
declare global {
    interface Array<T> extends ArrayExtensions<T> {
    }
    interface ArrayConstructor {
        ofSize<T = any>(length: number): T[];
        empty<T = any>(): T[];
    }
}
export {};
//# sourceMappingURL=array.d.ts.map